<?php

namespace Drupal\simple_entity_merge\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\ContentEntityConfirmFormBase;
use Drupal\Core\Url;
use Drupal\field\Entity\FieldStorageConfig;

/**
 * Form class to confirm the action of merging an entity.
 *
 * @package Drupal\simple_entity_merge\Form
 */
class Merge extends ContentEntityConfirmFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'simple_entity_merge_merge';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $entity = $this->getEntity();
    $form = [];
    $form['help'] = [
      '#markup' => $this->t('Use this form to replace all references to "@label" at once for another one. This functionality can be useful to merge two synonim taxonomy terms for example', [
        '@label' => $entity->label(),
      ]),
    ];
    $form['simple_entity_merge'] = [
      '#title' => $this->t('Replace all references to "@label" for:', [
        '@label' => $entity->label(),
      ]),
      '#required' => TRUE,
      '#type' => 'entity_autocomplete',
      '#target_type' => $entity->getEntityTypeId(),
      '#selection_settings' => [
        'target_bundles' => [$entity->bundle()],
      ],
    ];

    return $form + parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
    $entity = $this->getEntity();
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    $entity = $this->getEntity();
    $entity_merge_id = $form_state->getValue('simple_entity_merge');
    $fields = FieldStorageConfig::loadMultiple();
    foreach ($fields as $field_name => $field) {
      // The field is entity reference and the target is of the same type as
      // the entity we want to merge.
      if ($field->getType() == 'entity_reference' && $field->getSetting('target_type') == $entity->getEntityTypeId()) {
        $query = \Drupal::entityQuery($field->getTargetEntityTypeId());
        $query->condition($field->getName(), $entity->id());
        $ids = $query->execute();
        $entities = $this->entityTypeManager->getStorage($field->getTargetEntityTypeId())->loadMultiple($ids);
        foreach ($entities as $referencing_entity) {
          // Change the reference.
          $referencing_entity->{$field->getName()}->setValue(['target_id' => $entity_merge_id]);
          $referencing_entity->save();
        }
      }
    }
    drupal_set_message($this->t('All references to this content have been changed to the new one, now you can delete this one.'));
    $form_state->setRedirect('entity.' . $entity->getEntityTypeId() . '.edit_form', [$entity->getEntityTypeId() => $entity->id()]);

  }

  /**
   * Returns the question to ask the user for confirmation.
   *
   * @return string
   *   The form question. The page title will be set to this value.
   */
  public function getQuestion() {
    /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    $entity = $this->getEntity();
    return $this->t('Are you sure you want to merge "@label" with another entity', [
      '@label' => $entity->label(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    $entity = $this->getEntity();
    // On cancel go back to the entity.
    return Url::fromRoute('entity.' . $entity->getEntityTypeId() . '.canonical', [
      $entity->getEntityTypeId() => $this->getEntity()->id(),
    ]);
  }

}
